using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;
using NationalInstruments.UI;
using NationalInstruments.NI4882;

namespace harompaneles
{
    public partial class Form1 : Form
    {

        DateTime startTime;
        TimeSpan elapsedTime;

        string dataName1;
        string dataName2;
        string dataName3;
        string dataName4;
        string dataName5;
        string dataName6;

        double data1;
        double data2;
        double data3;
        double data4;
        double data5;
        double data6;

        StreamWriter fileWriter;

        Random rnd;

        NationalInstruments.NI4882.Device GPIBdev1;
        DSO Tektronix;
        public StepperMotor Motor1;

        double[,] ScopeCurve;

        public Form1()
        {
            InitializeComponent();
        }

        

        //  ################################################# Stepper motor ##################################################

        public class StepperMotor // class for handling the PD*-013-42 stepper motor
        {
            public SerialPort MotorSerialPort;
            public int MotorSpeed = 25000;
            public int MotorAccel = 5000;
            public bool MotorMoving = false;

            public System.Windows.Forms.Timer MotorTimer;

            // Constructor:
            public StepperMotor(SerialPort serial)
            {
                this.MotorSerialPort = serial;

            }

            public void Move(double seconds)
            {
                if (!this.MotorMoving)
                {
					this . MotorSerialPort . WriteLine ( "AV " + ( MotorSpeed * Math . Sign ( seconds ) ) . ToString ( ) ) ;
					this . MotorSerialPort . ReadLine ( ) ;
					//New part
					this . MotorMoving = true ;
					System . Threading . Thread . Sleep ( Convert . ToInt32 (2 * Math . Abs ( seconds ) * 1000 ) ) ;
					//New part ends
					this . Stop ( ) ;
					System . Threading . Thread . Sleep ( Convert . ToInt32 (4 * Math . Abs ( seconds ) * 1000 ) ) ;
                }
            }


            public void Stop()
            {
                this.MotorSerialPort.WriteLine("AV 0");
                this.MotorSerialPort.ReadLine();
                MotorMoving = false;

            }

            public void Stop(object sender, EventArgs e)
            {
                this.MotorSerialPort.WriteLine("AV 0");
                this.MotorSerialPort.ReadLine();

                MotorMoving = false;
                MotorTimer.Enabled = false;
            }

            public void Initialize()
            {
                this.MotorSerialPort.WriteLine("AA " + MotorAccel.ToString());
                this.MotorSerialPort.ReadLine();

                //MotorTimer.Tick += new EventHandler(this.Stop);
            }
        }

        //  ################################################# Stepper motor END ##################################################


        // ###################### Tektronix  ############################

        public class DSO // class for handling the Tektronix scope
        {
            public NationalInstruments.NI4882.Device GPIBdev;
            public int recordLength; // number of data points

            // Constructor:
            public DSO(NationalInstruments.NI4882.Device GPIBdev)
            {
                this.GPIBdev = GPIBdev;
            }

            public double[,] readScopeCurve(string Channel)
            {
                int i;
                string c;
                double voltsperdiv, vertpos, timeperdata;
                double[,] Curve = new double[2, this.recordLength];

                //read data:
                this.GPIBdev.Write("DATA:SOURCE " + Channel); // specify waveform data to transfer from the scope
                this.GPIBdev.Write("DATA:START 1"); // store data points from the 1st point in the record
                this.GPIBdev.Write("DATA:STOP " + this.recordLength.ToString()); // specify last data point in the record

                this.GPIBdev.Write(Channel + ":SCALE?"); // query the volts per div
                c = this.GPIBdev.ReadString();            // voltsperdiv
                voltsperdiv = Convert.ToDouble(c);

                this.GPIBdev.Write(Channel + ":POS?"); // e. g. "CH1:POS?", query the vertical position
                c = this.GPIBdev.ReadString();
                vertpos = Convert.ToDouble(c); // get vertpos in divisions
                vertpos = vertpos * voltsperdiv; // get vertpos in volts

                this.GPIBdev.Write("HOR:MAI?"); // query the time per div
                c = this.GPIBdev.ReadString();
                timeperdata = (double)Convert.ToDouble(c) / 100;

                this.GPIBdev.Write("CURVE?");
                byte[] header = this.GPIBdev.ReadByteArray(6); // header info e. g. "#42000" (2000 bytes, <2000> = 4 byte)
                byte[] curve = this.GPIBdev.ReadByteArray(this.recordLength * 1);
                this.GPIBdev.ReadByteArray(1); // read closing '\n' character

                // curve: "#<x><yyy><data><newline>"
                // <x>: number of y bytes, e.g. <yyy>=500, <x>=3

                double[] SC = new double[2];

                for (i = 0; i < curve.Length; i++)
                {
                    UInt32 l = Convert.ToUInt32((curve[i]));
                    SC[1] = (Convert.ToDouble(l) - 128) * 10 * voltsperdiv / 256 - vertpos;
                    // -128: shift to 0, 10: 10 divisions, 256: resolution (8 bit)
                    SC[0] = i * timeperdata;
                    Curve[0, i] = SC[0];
                    Curve[1, i] = SC[1];
                }

                return Curve;

            }

            public void Initialize()
            {

                // define data format: see page 2-22 in the Tektronix Programmer Manual
                this.GPIBdev.Write("DATA:ENCDG SRPBINARY"); // set waveform encoding to binary
                // S: swapped order, least significant byte is transferred first (doesn't matter when data width=1)
                // P: positive integer
                this.GPIBdev.Write("DATA:WIDTH 1"); // set number of bytes per data point (e.g. 1 byte/data point)
                // comment: in acquire mode 'Sample' only 1 byte is used, the least significant byte is filled with zeros
                this.GPIBdev.Write("HOR:RECO?"); // query the record length
                //System.Threading.Thread.Sleep(10);
                string c = this.GPIBdev.ReadString();
                this.recordLength = Convert.ToInt32(c);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            dataName1 = "time";
            dataName2 = "temperature";
            dataName3 = "resistance1";
            dataName4 = "resistance2";
            dataName5 = "empty";
            dataName6 = "empty";

            dataLabel1.Text = dataName1;
            dataLabel2.Text = dataName2;
            dataLabel3.Text = dataName3;
            dataLabel4.Text = dataName4;
            dataLabel5.Text = dataName5;
            dataLabel6.Text = dataName6;

            rnd = new Random();


            // Initializing GPIB devices

            byte port = 1;

            try
            {
                GPIBdev1 = new NationalInstruments.NI4882.Device(0, port);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                return;
            }


            // Initialize Tektronix

            Tektronix = new DSO(GPIBdev1);

            Tektronix.Initialize();

            //ScopeCurve = new double[2, Tektronix.recordLength];

            // Set filename for saving the data

            SaveFileDialog sfDialog = new SaveFileDialog();

            sfDialog.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            sfDialog.FilterIndex = 1;
            sfDialog.RestoreDirectory = true;

            //if (sfDialog.ShowDialog() == DialogResult.OK)
            //{

            //    fileWriter = new StreamWriter(sfDialog.FileName);
            //    fileWriter.AutoFlush = true;
            //    fileWriter.WriteLine(dataName1 + "\t" + dataName2 + "\t" + dataName3 + "\t" + dataName4 + "\t" + dataName5 + "\t" + dataName6);


            //}

            // todo: handle DialogResult.Cancel

            // listbox feltltse a tallt soros portokkal

            string[] ports = SerialPort.GetPortNames();
            foreach (string port1 in ports)
            {
                comList.Items.Add(port1);
            }


            
            
            timer1.Interval = Convert.ToInt32(TimerInterval.Text);
            timer1.Enabled = false;

            startTime = DateTime.Now;

        }

        private void timer1_Tick(object sender, EventArgs e)
        {

            double GraphPoint1X;
            double GraphPoint1Y;

            double GraphPoint2X;
            double GraphPoint2Y;

            double GraphPoint3X;
            double GraphPoint3Y;

            string response;

            elapsedTime = DateTime.Now - startTime;


            ScopeCurve = Tektronix.readScopeCurve("CH1");

            double[] xData = new double[Tektronix.recordLength];
            double[] yData = new double[Tektronix.recordLength];

            for (int k = 0; k < Tektronix.recordLength; k++)
            {
                xData[k] = ScopeCurve[0, k];
                yData[k] = ScopeCurve[1, k];
            }

            scatterGraph1.PlotXY(xData, yData);

            // plda az oszcilloszkppal val kommunikcira:

            //GPIBdev1.Write("CH1:SCALE?");
            //response = GPIBdev1.ReadString();
            //response = response.Substring(0, 13);

            data1 = elapsedTime.TotalSeconds;
            data2 = 0;
            data3 = 0;
            data4 = 0;
            data5 = data1;
            data6 = 0;

            //'Filling the data variables

            dataBox1.Text = data1.ToString("0.0");
            dataBox2.Text = data2.ToString();
            dataBox3.Text = data3.ToString();
            dataBox4.Text = data4.ToString();
            dataBox5.Text = data5.ToString();
            dataBox6.Text = data6.ToString();


            //'X and Y value of the new point on graph 1
            GraphPoint1X = data1;
            GraphPoint1Y = data2;

            //'X and Y value of the new point on graph 2
            GraphPoint2X = data2;
            GraphPoint2Y = data3;

            //'X and Y value of the new point on graph 3
            GraphPoint3X = data3;
            GraphPoint3Y = data4;

            //'Plotting the new points on the graph
            //CWGraph1.ChartXY GraphPoint1
            //CWGraph2.ChartXY GraphPoint2
            //CWGraph3.ChartXY GraphPoint3

            //scatterGraph1.PlotXYAppend(GraphPoint1X, GraphPoint1Y);
            //scatterGraph2.PlotXYAppend(GraphPoint2X, GraphPoint2Y);
            //scatterGraph3.PlotXYAppend(GraphPoint3X, GraphPoint3Y);



            //'Saving the data
            //SaveDataPoint();

            //timer1.Enabled = false;

        }

        private void switch1_StateChanged(object sender, NationalInstruments.UI.ActionEventArgs e)
        {
            switch (timer1.Enabled)
            {
                case true:
                    timer1.Enabled = false;
                    break;
                case false:
                    timer1.Enabled = true;
                    break;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            timer1.Interval = Convert.ToInt32(TimerInterval.Text);
        }

        private void SaveDataPoint()
        {
            fileWriter.WriteLine(data1.ToString("0.00") + "\t" + data2.ToString() + "\t" + data3.ToString() + "\t" +
            data4.ToString() + "\t" + data5.ToString() + "\t" + data6.ToString());

            fileWriter.Flush();

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (fileWriter != null)
            {
                try
                {
                    //fileWriter.Flush();
                    fileWriter.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString());
                }
            }
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }


        // ###################### graph 1 ############################

        private void graph1AutoY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph1AutoY.Checked)
            {
                case true:
                    scatterGraph1.Plots[0].YAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph1.Plots[0].YAxis.Mode = AxisMode.Fixed;

                    if (graph1MaxY.Value > graph1MinY.Value)
                        scatterGraph1.Plots[0].YAxis.Range = new Range(graph1MinY.Value, graph1MaxY.Value);
                    break;
            }

        }

        private void graph1MinY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph1AutoY_CheckedChanged(sender, e);
        }

        private void graph1MaxY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph1AutoY_CheckedChanged(sender, e);
        }

        private void graph1LogY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph1LogY.Checked)
            {
                case true:
                    scatterGraph1.Plots[0].YAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph1.Plots[0].YAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }

        private void graph1AutoX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph1AutoX.Checked)
            {
                case true:
                    scatterGraph1.Plots[0].XAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph1.Plots[0].XAxis.Mode = AxisMode.Fixed;

                    if (graph1MaxX.Value > graph1MinX.Value)
                        scatterGraph1.Plots[0].XAxis.Range = new Range(graph1MinX.Value, graph1MaxX.Value);
                    break;
            }
        }

        private void graph1MinX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph1AutoX_CheckedChanged(sender, e);
        }

        private void graph1MaxX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph1AutoX_CheckedChanged(sender, e);
        }

        private void graph1LogX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph1LogX.Checked)
            {
                case true:
                    scatterGraph1.Plots[0].XAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph1.Plots[0].XAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }


        // ###################### graph 2 ############################

        private void graph2AutoY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph2AutoY.Checked)
            {
                case true:
                    scatterGraph2.Plots[0].YAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph2.Plots[0].YAxis.Mode = AxisMode.Fixed;

                    if (graph2MaxY.Value > graph2MinY.Value)
                        scatterGraph2.Plots[0].YAxis.Range = new Range(graph2MinY.Value, graph2MaxY.Value);
                    break;
            }
        }

        private void graph2MinY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph2AutoY_CheckedChanged(sender, e);
        }

        private void graph2MaxY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph2AutoY_CheckedChanged(sender, e);
        }

        private void graph2LogY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph2LogY.Checked)
            {
                case true:
                    scatterGraph2.Plots[0].YAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph2.Plots[0].YAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }

        private void graph2AutoX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph2AutoX.Checked)
            {
                case true:
                    scatterGraph2.Plots[0].XAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph2.Plots[0].XAxis.Mode = AxisMode.Fixed;

                    if (graph2MaxX.Value > graph2MinX.Value)
                        scatterGraph2.Plots[0].XAxis.Range = new Range(graph2MinX.Value, graph2MaxX.Value);
                    break;
            }
        }

        private void graph2MinX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph2AutoX_CheckedChanged(sender, e);
        }

        private void graph2MaxX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph2AutoX_CheckedChanged(sender, e);
        }

        private void graph2LogX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph2LogX.Checked)
            {
                case true:
                    scatterGraph2.Plots[0].XAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph2.Plots[0].XAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }

        // ###################### graph 3 ############################

        private void graph3AutoY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph3AutoY.Checked)
            {
                case true:
                    scatterGraph3.Plots[0].YAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph3.Plots[0].YAxis.Mode = AxisMode.Fixed;

                    if (graph3MaxY.Value > graph3MinY.Value)
                        scatterGraph3.Plots[0].YAxis.Range = new Range(graph3MinY.Value, graph3MaxY.Value);
                    break;
            }
        }

        private void graph3MinY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph3AutoY_CheckedChanged(sender, e);
        }

        private void graph3MaxY_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph3AutoY_CheckedChanged(sender, e);
        }

        private void graph3LogY_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph3LogY.Checked)
            {
                case true:
                    scatterGraph3.Plots[0].YAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph3.Plots[0].YAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }

        private void graph3AutoX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph3AutoX.Checked)
            {
                case true:
                    scatterGraph3.Plots[0].XAxis.Mode = AxisMode.AutoScaleLoose;
                    break;
                case false:
                    scatterGraph3.Plots[0].XAxis.Mode = AxisMode.Fixed;

                    if (graph3MaxX.Value > graph3MinX.Value)
                        scatterGraph3.Plots[0].XAxis.Range = new Range(graph3MinX.Value, graph3MaxX.Value);
                    break;
            }
        }

        private void graph3MinX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph3AutoX_CheckedChanged(sender, e);
        }

        private void graph3MaxX_AfterChangeValue(object sender, AfterChangeNumericValueEventArgs e)
        {
            graph3AutoX_CheckedChanged(sender, e);
        }

        private void graph3LogX_CheckedChanged(object sender, EventArgs e)
        {
            switch (graph3LogX.Checked)
            {
                case true:
                    scatterGraph3.Plots[0].XAxis.ScaleType = ScaleType.Logarithmic;
                    break;
                case false:
                    scatterGraph3.Plots[0].XAxis.ScaleType = ScaleType.Linear;
                    break;
            }
        }

        private void groupBox4_Enter(object sender, EventArgs e)
        {

        }

        private void initMotorButton_Click(object sender, EventArgs e)
        {
            if (Serial1.IsOpen)
            {
                Serial1.Close();
            }

            // fixre llthat, pl. Serial1.PortName = "COM4"
            Serial1.PortName = comList.SelectedItem.ToString(); 

            Serial1.DataBits = 8;
            Serial1.StopBits = StopBits.One;
            Serial1.Parity = Parity.None;
            Serial1.BaudRate = 9600;
            Serial1.NewLine = "\r";
            Serial1.DtrEnable = true;
            //Serial1.RtsEnable = true;
            // Serial1.Encoding = Encoding.ASCII;
            Serial1.Handshake = Handshake.None;
            //Serial1.
            Serial1.Open();


            Motor1 = new StepperMotor(Serial1);

            Motor1.Initialize();

        }

        private void PanicButton_Click(object sender, EventArgs e)
        {
            Motor1.Stop();
            timer1.Enabled = false;
        }

        private void MotorStop_Click(object sender, EventArgs e)
        {
            Motor1.Stop();
        }

        private void motor_plus_Click(object sender, EventArgs e)
        {
            Motor1.Move(Convert.ToDouble(MoveSeconds.Text));
        }

        private void motor_minus_Click(object sender, EventArgs e)
        {
            Motor1.Move(-Convert.ToDouble(MoveSeconds.Text));
        }



    }
}